// ------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//     Runtime Version: 16.0.0.0
//  
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
// ------------------------------------------------------------------------------
namespace macro_utils_h_generator
{
    using System.Linq;
    using System.Text;
    using System.Collections.Generic;
    using System;
    
    /// <summary>
    /// Class to produce the template output
    /// </summary>
    
    #line 1 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
    [global::System.CodeDom.Compiler.GeneratedCodeAttribute("Microsoft.VisualStudio.TextTemplating", "16.0.0.0")]
    public partial class macro_utils_generated : macro_utils_generatedBase
    {
#line hidden
        /// <summary>
        /// Create the template output
        /// </summary>
        public virtual string TransformText()
        {
            this.Write(@"// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

/*THIS FILE IS GENERATED*/
/*DO NOT EDIT BY HAND!!!*/
/*instead edit macro_utils_generated.tt here: http://www.github.com/azure/azure-macro-utils-c.git */

");
            
            #line 14 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
/*CHANGE BELOW 2 VARIABLES TO GET MORE / LESS */ 
            
            #line default
            #line hidden
            
            #line 15 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
int nArithmetic=2048;
            
            #line default
            #line hidden
            
            #line 16 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
int nMacroParameters=124;/*127 parameters in one macro definition in C99 in chapter 5.2.4.1 Translation limits*/
            
            #line default
            #line hidden
            this.Write("\r\n#ifndef MACRO_UTILS_GENERATED_H\r\n#define MACRO_UTILS_GENERATED_H\r\n\r\n#define MU_" +
                    "IFCOMMA(N) MU_C2(MU_IFCOMMA_, N)\r\n#define MU_IFCOMMA_0\r\n#define MU_IFCOMMA_2\r\n");
            
            #line 24 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=4;i<=nMacroParameters;i+=2)
            
            #line default
            #line hidden
            
            #line 25 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_IFCOMMA_");
            
            #line 26 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(" ,\r\n");
            
            #line 27 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_IFCOMMALOGIC(N) MU_C2(MU_IFCOMMALOGIC_, N)\r\n#define MU_IFCOMMALOGIC_" +
                    "0 \r\n");
            
            #line 31 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=1;i<=nMacroParameters;i+=1)
            
            #line default
            #line hidden
            
            #line 32 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_IFCOMMALOGIC_");
            
            #line 33 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(" ,\r\n");
            
            #line 34 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_IFCOMMA_NOFIRST(N) MU_C2(MU_IFCOMMA_NOFIRST, N)\r\n#define MU_IFCOMMA_" +
                    "NOFIRST1 \r\n");
            
            #line 38 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=2;i<=nMacroParameters;i++)
            
            #line default
            #line hidden
            
            #line 39 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_IFCOMMA_NOFIRST");
            
            #line 40 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(" ,\r\n");
            
            #line 41 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_DEC(x) MU_C2(MU_DEC,x)\r\n");
            
            #line 44 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nArithmetic;i>=1;i--)
            
            #line default
            #line hidden
            
            #line 45 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_DEC");
            
            #line 46 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(" ");
            
            #line 46 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-1));
            
            #line default
            #line hidden
            this.Write("\r\n");
            
            #line 47 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_INC(x) MU_C2(MU_INC,x)\r\n");
            
            #line 50 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nArithmetic;i>=0;i--)
            
            #line default
            #line hidden
            
            #line 51 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_INC");
            
            #line 52 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(" ");
            
            #line 52 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i+1));
            
            #line default
            #line hidden
            this.Write("\r\n");
            
            #line 53 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_DIV2(x) MU_C2(MU_DIV2_,x)\r\n\r\n");
            
            #line 57 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nArithmetic;i>=0;i--)
            
            #line default
            #line hidden
            
            #line 58 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_DIV2_");
            
            #line 59 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(" ");
            
            #line 59 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i/2));
            
            #line default
            #line hidden
            this.Write("\r\n");
            
            #line 60 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n");
            
            #line 62 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
/*the below produces MOD2 for numbers*/
            
            #line default
            #line hidden
            this.Write("#define MU_MOD2(x) MU_C2(MU_MOD2_,x)\r\n");
            
            #line 64 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nArithmetic;i>=0;i--)
            
            #line default
            #line hidden
            
            #line 65 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_MOD2_");
            
            #line 66 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(" ");
            
            #line 66 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i%2));
            
            #line default
            #line hidden
            this.Write("\r\n");
            
            #line 67 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_THE_NTH_ARG(");
            
            #line 69 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=1;i<=nMacroParameters;i++)
            
            #line default
            #line hidden
            
            #line 70 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 70 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(", ");
            
            #line 70 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(" ... ) P");
            
            #line 70 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(nMacroParameters));
            
            #line default
            #line hidden
            this.Write("\r\n\r\n#ifdef _MSC_VER\r\n#define MU_COUNT_1_OR_MORE_ARG(...) MU_THE_NTH_ARG MU_LPAREN" +
                    " __VA_ARGS__, \\\r\n");
            
            #line 74 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nMacroParameters-1;i>=1;i--){
            
            #line default
            #line hidden
            
            #line 74 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i.ToString()+((i>1)?", ":"")));
            
            #line default
            #line hidden
            
            #line 74 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n#define MU_MORE_THAN_1_ARG(...) MU_THE_NTH_ARG MU_LPAREN __VA_ARGS__, ");
            
            #line 75 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nMacroParameters-2;i>=1;i--){
            
            #line default
            #line hidden
            
            #line 75 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(1));
            
            #line default
            #line hidden
            this.Write(",");
            
            #line 75 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("0)\r\n#else\r\n#define MU_COUNT_1_OR_MORE_ARG(...) MU_THE_NTH_ARG (__VA_ARGS__, \\\r\n");
            
            #line 78 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nMacroParameters-1;i>=1;i--){
            
            #line default
            #line hidden
            
            #line 78 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i.ToString()+((i>1)?", ":"")));
            
            #line default
            #line hidden
            
            #line 78 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n#define MU_MORE_THAN_1_ARG(...) MU_THE_NTH_ARG(__VA_ARGS__, ");
            
            #line 79 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nMacroParameters-2;i>=1;i--){
            
            #line default
            #line hidden
            
            #line 79 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(1));
            
            #line default
            #line hidden
            this.Write(",");
            
            #line 79 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(@" 0)
#endif

#define MU_COUNT_ARG(...) MU_C2(MU_COUNT_ARG_, MU_ISEMPTY(__VA_ARGS__))(__VA_ARGS__)
#define MU_COUNT_ARG_1(...) 0
#define MU_COUNT_ARG_0(...) MU_C1(MU_COUNT_1_OR_MORE_ARG(__VA_ARGS__))

#define MU_ISEMPTY(...) MU_C5(MU_DISPATCH_EMPTY_, MU_MORE_THAN_1_ARG(MU_TRIGGER_PARENTHESIS __VA_ARGS__ ()), MU_MORE_THAN_1_ARG(__VA_ARGS__), MU_MORE_THAN_1_ARG(__VA_ARGS__ ()), MU_MORE_THAN_1_ARG(MU_TRIGGER_PARENTHESIS __VA_ARGS__))
#define MU_DISPATCH_EMPTY_1000 1
#define MU_DISPATCH_EMPTY_0000 0
#define MU_DISPATCH_EMPTY_1100 0
#define MU_DISPATCH_EMPTY_1111 0
#define MU_DISPATCH_EMPTY_1001 0
#define MU_DISPATCH_EMPTY_1010 0


");
            
            #line 95 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nMacroParameters;i>=2;i--)
            
            #line default
            #line hidden
            
            #line 96 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_FOR_EACH_1_");
            
            #line 97 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 97 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 97 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i)?", ":"")));
            
            #line default
            #line hidden
            
            #line 97 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(") \\\r\nX(P1) \\\r\nMU_FOR_EACH_1_");
            
            #line 99 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-1));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 99 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=2;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 99 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i)?", ":"")));
            
            #line default
            #line hidden
            
            #line 99 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n\r\n");
            
            #line 101 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(@"#define MU_FOR_EACH_1_1(X, P1) \
X(P1)

#ifdef _MSC_VER
#define MU_FOR_EACH_1(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_1_,MU_C1(MU_COUNT_ARG(__VA_ARGS__))) MU_LPAREN MACRO_TO_INVOKE, __VA_ARGS__)
#else
#define MU_FOR_EACH_1(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_1_,MU_C1(MU_COUNT_ARG(__VA_ARGS__))) ( MACRO_TO_INVOKE, __VA_ARGS__)
#endif

");
            
            #line 111 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nMacroParameters;i>=2;i--)
            
            #line default
            #line hidden
            
            #line 112 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_FOR_EACH_1_KEEP_1_");
            
            #line 113 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(X, keep, ");
            
            #line 113 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 113 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+(j<i?", ":"")));
            
            #line default
            #line hidden
            
            #line 113 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(") \\\r\nX(keep, P1) \\\r\nMU_FOR_EACH_1_KEEP_1_");
            
            #line 115 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-1));
            
            #line default
            #line hidden
            this.Write("(X, keep, ");
            
            #line 115 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=2;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 115 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture((j).ToString()+(j<i?", ":"")));
            
            #line default
            #line hidden
            
            #line 115 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n\r\n\r\n");
            
            #line 118 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(@"
#define MU_FOR_EACH_1_KEEP_1_1(X, keep, P1) \
X(keep, P1)

#ifdef _MSC_VER
#define MU_FOR_EACH_1_KEEP_1(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_1_KEEP_1_, MU_C2(MU_DEC,MU_C1(MU_COUNT_ARG(__VA_ARGS__)))) MU_LPAREN MACRO_TO_INVOKE, __VA_ARGS__)
#else
#define MU_FOR_EACH_1_KEEP_1(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_1_KEEP_1_, MU_C2(MU_DEC,MU_C1(MU_COUNT_ARG(__VA_ARGS__)))) ( MACRO_TO_INVOKE, __VA_ARGS__)
#endif

");
            
            #line 129 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nMacroParameters;i>=2;i--)
            
            #line default
            #line hidden
            
            #line 130 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_FOR_EACH_1_KEEP_2_");
            
            #line 131 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(X, keep1, keep2, ");
            
            #line 131 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 131 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+(j<i?", ":"")));
            
            #line default
            #line hidden
            
            #line 131 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(") \\\r\nX(keep1, keep2, P1) \\\r\nMU_FOR_EACH_1_KEEP_2_");
            
            #line 133 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-1));
            
            #line default
            #line hidden
            this.Write("(X, keep1, keep2, ");
            
            #line 133 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=2;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 133 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture((j).ToString()+(j<i?", ":"")));
            
            #line default
            #line hidden
            
            #line 133 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n\r\n\r\n");
            
            #line 136 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(@"
#define MU_FOR_EACH_1_KEEP_2_1(X, keep1, keep2, P1) \
X(keep1, keep2, P1)

#define MU_FOR_EACH_1_KEEP_2_0(...)

#ifdef _MSC_VER
#define MU_FOR_EACH_1_KEEP_2(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_1_KEEP_2_, MU_C2(MU_DEC,MU_C2(MU_DEC,MU_C1(MU_COUNT_ARG(__VA_ARGS__))))) MU_LPAREN MACRO_TO_INVOKE, __VA_ARGS__)
#else
#define MU_FOR_EACH_1_KEEP_2(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_1_KEEP_2_, MU_C2(MU_DEC,MU_C2(MU_DEC,MU_C1(MU_COUNT_ARG(__VA_ARGS__))))) ( MACRO_TO_INVOKE, __VA_ARGS__)
#endif

");
            
            #line 149 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nMacroParameters;i>=4;i-=2)
            
            #line default
            #line hidden
            
            #line 150 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_FOR_EACH_2_KEEP_1_");
            
            #line 151 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(X, keep, ");
            
            #line 151 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 151 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+(j<i?", ":"")));
            
            #line default
            #line hidden
            
            #line 151 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(") \\\r\nX(keep, P1, P2) \\\r\nMU_FOR_EACH_2_KEEP_1_");
            
            #line 153 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-2));
            
            #line default
            #line hidden
            this.Write("(X, keep, ");
            
            #line 153 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=3;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 153 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture((j).ToString()+(j<i?", ":"")));
            
            #line default
            #line hidden
            
            #line 153 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n\r\n\r\n");
            
            #line 156 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(@"
#define MU_FOR_EACH_2_KEEP_1_1(...)

#define MU_FOR_EACH_2_KEEP_1_0(...)

#define MU_FOR_EACH_2_KEEP_1_2(X, keep, P1, P2) \
    X(keep, P1, P2) \

#ifdef _MSC_VER
#define MU_FOR_EACH_2_KEEP_1(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_2_KEEP_1_, MU_C2(MU_DEC,MU_C1(MU_COUNT_ARG(__VA_ARGS__)))) MU_LPAREN MACRO_TO_INVOKE, __VA_ARGS__)
#else
#define MU_FOR_EACH_2_KEEP_1(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_2_KEEP_1_, MU_C2(MU_DEC,MU_C1(MU_COUNT_ARG(__VA_ARGS__)))) ( MACRO_TO_INVOKE, __VA_ARGS__)
#endif


");
            
            #line 172 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=nMacroParameters;i>=4;i-=2)
            
            #line default
            #line hidden
            
            #line 173 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
{
            
            #line default
            #line hidden
            this.Write("#define MU_FOR_EACH_2_KEEP_2_");
            
            #line 174 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(X, keep1, keep2, ");
            
            #line 174 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 174 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+(j<i?", ":"")));
            
            #line default
            #line hidden
            
            #line 174 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(") \\\r\nX(keep1, keep2, P1, P2) \\\r\nMU_FOR_EACH_2_KEEP_2_");
            
            #line 176 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-2));
            
            #line default
            #line hidden
            this.Write("(X, keep1, keep2, ");
            
            #line 176 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=3;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 176 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture((j).ToString()+(j<i?", ":"")));
            
            #line default
            #line hidden
            
            #line 176 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n\r\n\r\n");
            
            #line 179 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(@"
#define MU_FOR_EACH_2_KEEP_2_1(...)

#define MU_FOR_EACH_2_KEEP_2_0(...)

#define MU_FOR_EACH_2_KEEP_2_2(X, keep1, keep2, P1, P2) \
    X(keep1, keep2, P1, P2) \

#ifdef _MSC_VER
#define MU_FOR_EACH_2_KEEP_2(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_2_KEEP_2_, MU_C2(MU_DEC,MU_C2(MU_DEC,MU_C1(MU_COUNT_ARG(__VA_ARGS__))))) MU_LPAREN MACRO_TO_INVOKE, __VA_ARGS__)
#else
#define MU_FOR_EACH_2_KEEP_2(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_2_KEEP_2_, MU_C2(MU_DEC, MU_C2(MU_DEC,MU_C1(MU_COUNT_ARG(__VA_ARGS__))))) ( MACRO_TO_INVOKE, __VA_ARGS__)
#endif


#define MU_FOR_EACH_2_0(...)

#define MU_FOR_EACH_2_2(X, P1, P2) \
X(P1, P2)

");
            
            #line 200 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=4;i<=nMacroParameters;i+=2) {
            
            #line default
            #line hidden
            this.Write("#define MU_FOR_EACH_2_");
            
            #line 201 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 201 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 201 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i)?", ":"")));
            
            #line default
            #line hidden
            
            #line 201 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(") \\\r\nX(P1, P2) \\\r\nMU_FOR_EACH_2_");
            
            #line 203 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-2));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 203 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=3;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 203 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i)?", ":"")));
            
            #line default
            #line hidden
            
            #line 203 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n\r\n");
            
            #line 205 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_FOR_EACH_2_REVERSE_0(...)\r\n\r\n#define MU_FOR_EACH_2_REVERSE_2(X, P1, " +
                    "P2) \\\r\nX(P1, P2)\r\n\r\n");
            
            #line 212 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=4;i<=nMacroParameters;i+=2) {
            
            #line default
            #line hidden
            this.Write("#define MU_FOR_EACH_2_REVERSE_");
            
            #line 213 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 213 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 213 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i)?", ":"")));
            
            #line default
            #line hidden
            
            #line 213 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(") \\\r\nX(P");
            
            #line 214 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-1));
            
            #line default
            #line hidden
            this.Write(", P");
            
            #line 214 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(") \\\r\nMU_FOR_EACH_2_REVERSE_");
            
            #line 215 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-2));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 215 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i-2;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 215 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i-2)?", ":"")));
            
            #line default
            #line hidden
            
            #line 215 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n\r\n");
            
            #line 217 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_FOR_EACH_1_COUNTED_0(...)\r\n\r\n#define MU_FOR_EACH_1_COUNTED_1(X, P1) " +
                    "\\\r\n    X(1, P1)\r\n\r\n");
            
            #line 224 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=2;i<=nMacroParameters;i++) {
            
            #line default
            #line hidden
            this.Write("#define MU_FOR_EACH_1_COUNTED_");
            
            #line 225 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 225 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 225 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i)?", ":"")));
            
            #line default
            #line hidden
            
            #line 225 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(") \\\r\nX(");
            
            #line 226 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(", P1) \\\r\nMU_FOR_EACH_1_COUNTED_");
            
            #line 227 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-1));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 227 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=2;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 227 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i)?", ":"")));
            
            #line default
            #line hidden
            
            #line 227 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n\r\n");
            
            #line 229 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_FOR_EACH_2_COUNTED_0(...)\r\n\r\n#define MU_FOR_EACH_2_COUNTED_2(X, P1, " +
                    "P2) \\\r\n    X(2, P1, P2)\r\n\r\n");
            
            #line 236 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=4;i<=nMacroParameters;i+=2) {
            
            #line default
            #line hidden
            this.Write("#define MU_FOR_EACH_2_COUNTED_");
            
            #line 237 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 237 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=1;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 237 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i)?", ":"")));
            
            #line default
            #line hidden
            
            #line 237 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(") \\\r\nX(");
            
            #line 238 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(", P1, P2) \\\r\nMU_FOR_EACH_2_COUNTED_");
            
            #line 239 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-2));
            
            #line default
            #line hidden
            this.Write("(X, ");
            
            #line 239 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var j=3;j<=i;j++){
            
            #line default
            #line hidden
            this.Write("P");
            
            #line 239 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(j.ToString()+((j<i)?", ":"")));
            
            #line default
            #line hidden
            
            #line 239 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write(")\r\n\r\n");
            
            #line 241 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#ifdef _MSC_VER\r\n#define MU_FOR_EACH_2(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_" +
                    "2_, MU_C1(MU_COUNT_ARG(__VA_ARGS__))) MU_LPAREN MACRO_TO_INVOKE, __VA_ARGS__)\r\n/" +
                    "*the COUNTED breed of MU_FOR_EACH macro invokes a macro with 3 parameters: 1st b" +
                    "eing the count of invocation. For example.\r\nMU_FOR_EACH_2_COUNTER(MACRO, a,b,c,d" +
                    ",e,f) will result in \r\nMACRO(6, a,b)\r\nMACRO(4, c,d)\r\nMACRO(2, e,f)\r\nThis macro e" +
                    "xists because we need a \"stop condition\" in outputting COMMA... when calling a f" +
                    "unction f(a,b,c,d) cannot be f(a,b,c,d,) <=doesn\'t compile (as opposed to enum d" +
                    "efinition)\r\n*/\r\n#define MU_FOR_EACH_2_COUNTED(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR" +
                    "_EACH_2_COUNTED_, MU_C1(MU_COUNT_ARG(__VA_ARGS__))) MU_LPAREN MACRO_TO_INVOKE, _" +
                    "_VA_ARGS__)\r\n#define MU_FOR_EACH_1_COUNTED(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EA" +
                    "CH_1_COUNTED_, MU_C1(MU_COUNT_ARG(__VA_ARGS__))) MU_LPAREN MACRO_TO_INVOKE, __VA" +
                    "_ARGS__)\r\n\r\n/*MU_FOR_EACH_2_REVERSE acts just like MU_FOR_EACH_2, but in reverse" +
                    " order. Example:\r\nMU_FOR_EACH_2_REVERSE(X,a,b,c,d,e,f) => X(e,f) X(c,d) X (a, b)" +
                    " in this order */\r\n#define MU_FOR_EACH_2_REVERSE(MACRO_TO_INVOKE, ...) MU_C2(MU_" +
                    "FOR_EACH_2_REVERSE_, MU_C1(MU_COUNT_ARG(__VA_ARGS__))) MU_LPAREN MACRO_TO_INVOKE" +
                    ", __VA_ARGS__)\r\n#else\r\n#define MU_FOR_EACH_2(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_" +
                    "EACH_2_, MU_C1(MU_COUNT_ARG(__VA_ARGS__))) ( MACRO_TO_INVOKE, __VA_ARGS__)\r\n#def" +
                    "ine MU_FOR_EACH_2_COUNTED(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_2_COUNTED_, MU" +
                    "_C1(MU_COUNT_ARG(__VA_ARGS__))) ( MACRO_TO_INVOKE,  __VA_ARGS__)\r\n#define MU_FOR" +
                    "_EACH_1_COUNTED(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_1_COUNTED_, MU_C1(MU_COU" +
                    "NT_ARG(__VA_ARGS__))) ( MACRO_TO_INVOKE,  __VA_ARGS__)\r\n#define MU_FOR_EACH_2_RE" +
                    "VERSE(MACRO_TO_INVOKE, ...) MU_C2(MU_FOR_EACH_2_REVERSE_, MU_C1(MU_COUNT_ARG(__V" +
                    "A_ARGS__))) ( MACRO_TO_INVOKE, __VA_ARGS__)\r\n#endif\r\n\r\n#ifdef _MSC_VER\r\n#define " +
                    "MU_EXPAND_OR_MU_C1(x) x\r\n#else\r\n#define MU_EXPAND_OR_MU_C1(...) __VA_ARGS__\r\n#en" +
                    "dif\r\n\r\n#define MU_EXPAND_ARGS(...) __VA_ARGS__\r\n#define MU_EXPAND_TWICE(...) MU_" +
                    "EXPAND_ARGS(__VA_ARGS__)\r\n\r\n#define MU_DO_0(MACRO, ...) \\\r\nMACRO(0, __VA_ARGS__)" +
                    "\r\n\r\n");
            
            #line 277 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
for(var i=1;i<=nArithmetic;i++){
            
            #line default
            #line hidden
            this.Write("#define MU_DO_");
            
            #line 278 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write("(MACRO, ...) \\\r\nMACRO(");
            
            #line 279 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i));
            
            #line default
            #line hidden
            this.Write(", __VA_ARGS__) \\\r\nMU_DO_");
            
            #line 280 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
            this.Write(this.ToStringHelper.ToStringWithCulture(i-1));
            
            #line default
            #line hidden
            this.Write("(MACRO, __VA_ARGS__)\r\n\r\n\r\n");
            
            #line 283 "G:\w\Azure-MessagingStore\deps\azure-macro-utils-c\macro_utils_h_generator\macro_utils_generated.tt"
}
            
            #line default
            #line hidden
            this.Write("\r\n#define MU_DO(TIMES, MACRO, ...) MU_C2(MU_DO_, TIMES)(MACRO, __VA_ARGS__)\r\n\r\n#e" +
                    "ndif /*MACRO_UTILS_GENERATED_H*/\r\n");
            return this.GenerationEnvironment.ToString();
        }
    }
    
    #line default
    #line hidden
    #region Base class
    /// <summary>
    /// Base class for this transformation
    /// </summary>
    [global::System.CodeDom.Compiler.GeneratedCodeAttribute("Microsoft.VisualStudio.TextTemplating", "16.0.0.0")]
    public class macro_utils_generatedBase
    {
        #region Fields
        private global::System.Text.StringBuilder generationEnvironmentField;
        private global::System.CodeDom.Compiler.CompilerErrorCollection errorsField;
        private global::System.Collections.Generic.List<int> indentLengthsField;
        private string currentIndentField = "";
        private bool endsWithNewline;
        private global::System.Collections.Generic.IDictionary<string, object> sessionField;
        #endregion
        #region Properties
        /// <summary>
        /// The string builder that generation-time code is using to assemble generated output
        /// </summary>
        protected System.Text.StringBuilder GenerationEnvironment
        {
            get
            {
                if ((this.generationEnvironmentField == null))
                {
                    this.generationEnvironmentField = new global::System.Text.StringBuilder();
                }
                return this.generationEnvironmentField;
            }
            set
            {
                this.generationEnvironmentField = value;
            }
        }
        /// <summary>
        /// The error collection for the generation process
        /// </summary>
        public System.CodeDom.Compiler.CompilerErrorCollection Errors
        {
            get
            {
                if ((this.errorsField == null))
                {
                    this.errorsField = new global::System.CodeDom.Compiler.CompilerErrorCollection();
                }
                return this.errorsField;
            }
        }
        /// <summary>
        /// A list of the lengths of each indent that was added with PushIndent
        /// </summary>
        private System.Collections.Generic.List<int> indentLengths
        {
            get
            {
                if ((this.indentLengthsField == null))
                {
                    this.indentLengthsField = new global::System.Collections.Generic.List<int>();
                }
                return this.indentLengthsField;
            }
        }
        /// <summary>
        /// Gets the current indent we use when adding lines to the output
        /// </summary>
        public string CurrentIndent
        {
            get
            {
                return this.currentIndentField;
            }
        }
        /// <summary>
        /// Current transformation session
        /// </summary>
        public virtual global::System.Collections.Generic.IDictionary<string, object> Session
        {
            get
            {
                return this.sessionField;
            }
            set
            {
                this.sessionField = value;
            }
        }
        #endregion
        #region Transform-time helpers
        /// <summary>
        /// Write text directly into the generated output
        /// </summary>
        public void Write(string textToAppend)
        {
            if (string.IsNullOrEmpty(textToAppend))
            {
                return;
            }
            // If we're starting off, or if the previous text ended with a newline,
            // we have to append the current indent first.
            if (((this.GenerationEnvironment.Length == 0) 
                        || this.endsWithNewline))
            {
                this.GenerationEnvironment.Append(this.currentIndentField);
                this.endsWithNewline = false;
            }
            // Check if the current text ends with a newline
            if (textToAppend.EndsWith(global::System.Environment.NewLine, global::System.StringComparison.CurrentCulture))
            {
                this.endsWithNewline = true;
            }
            // This is an optimization. If the current indent is "", then we don't have to do any
            // of the more complex stuff further down.
            if ((this.currentIndentField.Length == 0))
            {
                this.GenerationEnvironment.Append(textToAppend);
                return;
            }
            // Everywhere there is a newline in the text, add an indent after it
            textToAppend = textToAppend.Replace(global::System.Environment.NewLine, (global::System.Environment.NewLine + this.currentIndentField));
            // If the text ends with a newline, then we should strip off the indent added at the very end
            // because the appropriate indent will be added when the next time Write() is called
            if (this.endsWithNewline)
            {
                this.GenerationEnvironment.Append(textToAppend, 0, (textToAppend.Length - this.currentIndentField.Length));
            }
            else
            {
                this.GenerationEnvironment.Append(textToAppend);
            }
        }
        /// <summary>
        /// Write text directly into the generated output
        /// </summary>
        public void WriteLine(string textToAppend)
        {
            this.Write(textToAppend);
            this.GenerationEnvironment.AppendLine();
            this.endsWithNewline = true;
        }
        /// <summary>
        /// Write formatted text directly into the generated output
        /// </summary>
        public void Write(string format, params object[] args)
        {
            this.Write(string.Format(global::System.Globalization.CultureInfo.CurrentCulture, format, args));
        }
        /// <summary>
        /// Write formatted text directly into the generated output
        /// </summary>
        public void WriteLine(string format, params object[] args)
        {
            this.WriteLine(string.Format(global::System.Globalization.CultureInfo.CurrentCulture, format, args));
        }
        /// <summary>
        /// Raise an error
        /// </summary>
        public void Error(string message)
        {
            System.CodeDom.Compiler.CompilerError error = new global::System.CodeDom.Compiler.CompilerError();
            error.ErrorText = message;
            this.Errors.Add(error);
        }
        /// <summary>
        /// Raise a warning
        /// </summary>
        public void Warning(string message)
        {
            System.CodeDom.Compiler.CompilerError error = new global::System.CodeDom.Compiler.CompilerError();
            error.ErrorText = message;
            error.IsWarning = true;
            this.Errors.Add(error);
        }
        /// <summary>
        /// Increase the indent
        /// </summary>
        public void PushIndent(string indent)
        {
            if ((indent == null))
            {
                throw new global::System.ArgumentNullException("indent");
            }
            this.currentIndentField = (this.currentIndentField + indent);
            this.indentLengths.Add(indent.Length);
        }
        /// <summary>
        /// Remove the last indent that was added with PushIndent
        /// </summary>
        public string PopIndent()
        {
            string returnValue = "";
            if ((this.indentLengths.Count > 0))
            {
                int indentLength = this.indentLengths[(this.indentLengths.Count - 1)];
                this.indentLengths.RemoveAt((this.indentLengths.Count - 1));
                if ((indentLength > 0))
                {
                    returnValue = this.currentIndentField.Substring((this.currentIndentField.Length - indentLength));
                    this.currentIndentField = this.currentIndentField.Remove((this.currentIndentField.Length - indentLength));
                }
            }
            return returnValue;
        }
        /// <summary>
        /// Remove any indentation
        /// </summary>
        public void ClearIndent()
        {
            this.indentLengths.Clear();
            this.currentIndentField = "";
        }
        #endregion
        #region ToString Helpers
        /// <summary>
        /// Utility class to produce culture-oriented representation of an object as a string.
        /// </summary>
        public class ToStringInstanceHelper
        {
            private System.IFormatProvider formatProviderField  = global::System.Globalization.CultureInfo.InvariantCulture;
            /// <summary>
            /// Gets or sets format provider to be used by ToStringWithCulture method.
            /// </summary>
            public System.IFormatProvider FormatProvider
            {
                get
                {
                    return this.formatProviderField ;
                }
                set
                {
                    if ((value != null))
                    {
                        this.formatProviderField  = value;
                    }
                }
            }
            /// <summary>
            /// This is called from the compile/run appdomain to convert objects within an expression block to a string
            /// </summary>
            public string ToStringWithCulture(object objectToConvert)
            {
                if ((objectToConvert == null))
                {
                    throw new global::System.ArgumentNullException("objectToConvert");
                }
                System.Type t = objectToConvert.GetType();
                System.Reflection.MethodInfo method = t.GetMethod("ToString", new System.Type[] {
                            typeof(System.IFormatProvider)});
                if ((method == null))
                {
                    return objectToConvert.ToString();
                }
                else
                {
                    return ((string)(method.Invoke(objectToConvert, new object[] {
                                this.formatProviderField })));
                }
            }
        }
        private ToStringInstanceHelper toStringHelperField = new ToStringInstanceHelper();
        /// <summary>
        /// Helper to produce culture-oriented representation of an object as a string
        /// </summary>
        public ToStringInstanceHelper ToStringHelper
        {
            get
            {
                return this.toStringHelperField;
            }
        }
        #endregion
    }
    #endregion
}
