// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

#ifndef TEST_DEPENDENCY_NO_ENABLE_MOCKS_H
#define TEST_DEPENDENCY_NO_ENABLE_MOCKS_H

#include "umock_c/umock_c_prod.h"

#ifdef __cplusplus
extern "C" {
#endif

MOCKABLE_FUNCTION(, int, test_dependency_no_args_no_ENABLE_MOCKS);

#ifdef __cplusplus
}
#endif

#endif /* TEST_DEPENDENCY_NO_ENABLE_MOCKS_H */
