// Copyright (c) Microsoft. All rights reserved.

#define constbuffer_array_create real_constbuffer_array_create
#define constbuffer_array_create_with_move_buffers real_constbuffer_array_create_with_move_buffers
#define constbuffer_array_create_empty real_constbuffer_array_create_empty
#define constbuffer_array_create_from_array_array real_constbuffer_array_create_from_array_array
#define constbuffer_array_inc_ref real_constbuffer_array_inc_ref
#define constbuffer_array_dec_ref real_constbuffer_array_dec_ref
#define constbuffer_array_add_front real_constbuffer_array_add_front
#define constbuffer_array_remove_front real_constbuffer_array_remove_front
#define constbuffer_array_get_buffer_count real_constbuffer_array_get_buffer_count
#define constbuffer_array_get_buffer real_constbuffer_array_get_buffer
#define constbuffer_array_get_buffer_content real_constbuffer_array_get_buffer_content
#define constbuffer_array_get_all_buffers_size real_constbuffer_array_get_all_buffers_size
#define constbuffer_array_get_const_buffer_handle_array real_constbuffer_array_get_const_buffer_handle_array
#define CONSTBUFFER_ARRAY_HANDLE_contain_same real_CONSTBUFFER_ARRAY_HANDLE_contain_same