// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

#include "real_constbuffer_renames.h"

#define GBALLOC_H

#include "constbuffer.c"
