// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

#ifndef UMOCKTYPENAME_H
#define UMOCKTYPENAME_H

#ifdef __cplusplus
extern "C" {
#endif

char* umocktypename_normalize(const char* type_name);

#ifdef __cplusplus
}
#endif

#endif /* UMOCKTYPENAME_H */
