LIBRARY aziotsharedutil_dll
EXPORTS
    BUFFER_append
    BUFFER_append_build
    BUFFER_build
    BUFFER_clone
    BUFFER_content
    BUFFER_create
    BUFFER_delete
    BUFFER_enlarge
    BUFFER_length
    BUFFER_new
    BUFFER_pre_build
    BUFFER_prepend
    BUFFER_shrink
    BUFFER_size
    BUFFER_u_char
    BUFFER_unbuild

    Azure_Base64_Decode
    Azure_Base64_Encode
    Azure_Base64_Encode_Bytes
    Azure_Base32_Decode
    Azure_Base32_Decode_String
    Azure_Base32_Encode
    Azure_Base32_Encode_Bytes

    COND_RESULTStringStorage
    COND_RESULTStrings
    COND_RESULT_FromString
    CONSTBUFFER_Create
    CONSTBUFFER_CreateFromBuffer
    CONSTBUFFER_DecRef
    CONSTBUFFER_GetContent
    CONSTBUFFER_IncRef
    CONSTMAP_RESULTStringStorage
    CONSTMAP_RESULTStrings
    CONSTMAP_RESULT_FromString
    Condition_Deinit
    Condition_Init
    Condition_Post
    Condition_Wait
    ConstMap_Clone
    ConstMap_CloneWriteable
    ConstMap_ContainsKey
    ConstMap_ContainsValue
    ConstMap_Create
    ConstMap_Destroy
    ConstMap_GetInternals
    ConstMap_GetValue

    DList_AppendTailList
    DList_InitializeListHead
    DList_InsertHeadList
    DList_InsertTailList
    DList_IsListEmpty
    DList_RemoveEntryList
    DList_RemoveHeadList

    environment_get_variable
    HMACSHA256_ComputeHash
    Lock
    Lock_Deinit
    Lock_Init
    MAP_RESULTStringStorage
    MAP_RESULTStrings
    MAP_RESULT_FromString
    Map_Add
    Map_AddOrUpdate
    Map_Clone
    Map_ContainsKey
    Map_ContainsValue
    Map_Create
    Map_Delete
    Map_Destroy
    Map_GetInternals
    Map_GetValueFromKey
    Map_ToJSON
    OptionHandler_AddOption
    OptionHandler_Clone
    OptionHandler_Create
    OptionHandler_Destroy
    OptionHandler_FeedOptions
    SASToken_Create
    SASToken_CreateString
    SASToken_Validate
    SHA1FinalBits
    SHA1Input
    SHA1Reset
    SHA1Result
    SHA224FinalBits
    SHA224Input
    SHA224Reset
    SHA224Result
    SHA256FinalBits
    SHA256Input
    SHA256Reset
    SHA256Result
    SHA384FinalBits
    SHA384Input
    SHA384Reset
    SHA384Result
    SHA512FinalBits
    SHA512Input
    SHA512Reset
    SHA512Result
    STRING_TOKENIZER_create
    STRING_TOKENIZER_create_from_char
    STRING_TOKENIZER_destroy
    STRING_TOKENIZER_get_next_token
    STRING_c_str
    STRING_clone
    STRING_compare
    STRING_concat
    STRING_concat_with_STRING
    STRING_construct
    STRING_construct_n
    STRING_construct_sprintf
    STRING_copy
    STRING_copy_n
    STRING_delete
    STRING_empty
    STRING_from_byte_array
    STRING_length
    STRING_new
    STRING_new_JSON
    STRING_new_quoted
    STRING_new_with_memory
    STRING_quote
    STRING_sprintf
    STRING_replace
    THREADAPI_RESULTStringStorage
    THREADAPI_RESULTStrings
    THREADAPI_RESULT_FromString
    TLSIO_STATE_FromString
    TLSIO_STATEStrings
    ThreadAPI_Create
    ThreadAPI_Exit
    ThreadAPI_Join
    ThreadAPI_Sleep
    UNIQUEID_RESULTStringStorage
    UNIQUEID_RESULTStrings
    UNIQUEID_RESULT_FromString
    URL_Encode
    URL_EncodeString
    URL_Decode
    URL_DecodeString
    USHABlockSize
    USHAFinalBits
    USHAHashSize
    USHAHashSizeBits
    USHAInput
    USHAReset
    USHAResult
    UniqueId_Generate
    Unlock
    UUID_generate
    UUID_from_string
    UUID_to_string
    VECTOR_back
    VECTOR_clear
    VECTOR_create
    VECTOR_destroy
    VECTOR_element
    VECTOR_erase
    VECTOR_find_if
    VECTOR_front
    VECTOR_move
    VECTOR_push_back
    VECTOR_size
    connectionstringparser_parse
    connectionstringparser_parse_from_char
    connectionstringparser_splitHostName
    connectionstringparser_splitHostName_from_char
    consolelogger_log
    consolelogger_log_with_GetLastError
    gb_rand
    gballoc_calloc
    gballoc_deinit
    gballoc_free
    gballoc_getCurrentMemoryUsed
    gballoc_getMaximumMemoryUsed
    gballoc_init
    gballoc_malloc
    gballoc_realloc
    gbnetwork_init
    gbnetwork_deinit
    get_ctime
    get_difftime
    get_gmtime
    get_mktime
    get_time
    global_log_function
    hmac
    hmacFinalBits
    hmacInput
    hmacReset
    hmacResult
    http_proxy_io_get_interface_description
    mallocAndStrcpy_s
    platform_deinit
    platform_get_default_tlsio
    platform_get_platform_info
    platform_init
    singlylinkedlist_add
    singlylinkedlist_add_head
    singlylinkedlist_create
    singlylinkedlist_destroy
    singlylinkedlist_find
    singlylinkedlist_get_head_item
    singlylinkedlist_get_next_item
    singlylinkedlist_item_get_value
    singlylinkedlist_remove
    singlylinkedlist_remove_if
    singlylinkedlist_foreach
    size_tToString
    socketio_close
    socketio_create
    socketio_destroy
    socketio_dowork
    socketio_get_interface_description
    socketio_open
    socketio_send
    socketio_setoption

    tickcounter_create
    tickcounter_destroy
    tickcounter_get_current_ms

    tlsio_schannel_close
    tlsio_schannel_create
    tlsio_schannel_destroy
    tlsio_schannel_dowork
    tlsio_schannel_get_interface_description
    tlsio_schannel_open
    tlsio_schannel_send
    tlsio_schannel_setoption
    unsignedIntToString

    x509_schannel_create
    x509_schannel_destroy
    x509_schannel_get_certificate_context

    xio_close
    xio_create
    xio_destroy
    xio_dowork
    xio_open
    xio_retrieveoptions
    xio_send
    xio_setoption

    xlogging_get_log_function
    xlogging_get_log_function_GetLastError
    xlogging_set_log_function
    xlogging_set_log_function_GetLastError
    xlogging_LogErrorWinHTTPWithGetLastErrorAsStringFormatter
