/*Copyright (c) Microsoft Corporation.  All rights reserved.

Licensed under the Apache License, Version 2.0 (the ""License"");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER
EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,
FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.*/
//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by UnitTestsGenericResource.rc
//
#define VER_DEBUG                       0
#define VER_PRERELEASE                  0
#define _USE_DECLSPECS_FOR_SAL          0
#define _USE_ATTRIBUTES_FOR_SAL         0
#define __drv_typeConst                 0
#define WINAPI_PARTITION_DESKTOP        0x00000001
#define __drv_typeCond                  1
#define VS_VERSION_INFO                 1
#define VFFF_ISSHAREDFILE               0x0001
#define VFF_CURNEDEST                   0x0001
#define VIFF_FORCEINSTALL               0x0001
#define VER_PRODUCTMINORVERSION         2
#define WINAPI_PARTITION_APP            0x00000002
#define __drv_typeBitset                2
#define VFF_FILEINUSE                   0x0002
#define VIFF_DONTDELETEOLD              0x0002
#define __drv_typeExpr                  3
#define VFF_BUFFTOOSMALL                0x0004
#define VER_PRODUCTMAJORVERSION         6
#define _SAL_VERSION                    20
#define VS_USER_DEFINED                 100
#define VER_PRODUCTVERSION_W            0x0602
#define VER_PRODUCTBUILD                9200
#define VER_PRODUCTBUILD_QFE            16384
#define VER_PACKAGEBUILD_QFE            16384

// Next default values for new objects
//
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        101
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1000
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
