//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//     Runtime Version:4.0.30319.42000
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------

using System.Xml.Serialization;

// 
// This source code was auto-generated by xsd, Version=4.0.30319.33440.
// 


/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Diagnostics.DebuggerStepThroughAttribute()]
[System.ComponentModel.DesignerCategoryAttribute("code")]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
[System.Xml.Serialization.XmlRootAttribute(Namespace="http://tempuri.org/amqp_definitions", IsNullable=false)]
public partial class amqp {
    
    private object[] itemsField;
    
    private string nameField;
    
    private string labelField;
    
    /// <remarks/>
    [System.Xml.Serialization.XmlElementAttribute("doc", typeof(doc))]
    [System.Xml.Serialization.XmlElementAttribute("section", typeof(section))]
    public object[] Items {
        get {
            return this.itemsField;
        }
        set {
            this.itemsField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string name {
        get {
            return this.nameField;
        }
        set {
            this.nameField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string label {
        get {
            return this.labelField;
        }
        set {
            this.labelField = value;
        }
    }
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Diagnostics.DebuggerStepThroughAttribute()]
[System.ComponentModel.DesignerCategoryAttribute("code")]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
[System.Xml.Serialization.XmlRootAttribute(Namespace="http://tempuri.org/amqp_definitions", IsNullable=false)]
public partial class doc {
    
    private string nameField;
    
    private string titleField;
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string name {
        get {
            return this.nameField;
        }
        set {
            this.nameField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string title {
        get {
            return this.titleField;
        }
        set {
            this.titleField = value;
        }
    }
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Diagnostics.DebuggerStepThroughAttribute()]
[System.ComponentModel.DesignerCategoryAttribute("code")]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
[System.Xml.Serialization.XmlRootAttribute(Namespace="http://tempuri.org/amqp_definitions", IsNullable=false)]
public partial class section {
    
    private object[] itemsField;
    
    private string nameField;
    
    private string titleField;
    
    private string labelField;
    
    /// <remarks/>
    [System.Xml.Serialization.XmlElementAttribute("definition", typeof(definition))]
    [System.Xml.Serialization.XmlElementAttribute("doc", typeof(doc))]
    [System.Xml.Serialization.XmlElementAttribute("type", typeof(type))]
    public object[] Items {
        get {
            return this.itemsField;
        }
        set {
            this.itemsField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string name {
        get {
            return this.nameField;
        }
        set {
            this.nameField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string title {
        get {
            return this.titleField;
        }
        set {
            this.titleField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string label {
        get {
            return this.labelField;
        }
        set {
            this.labelField = value;
        }
    }
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Diagnostics.DebuggerStepThroughAttribute()]
[System.ComponentModel.DesignerCategoryAttribute("code")]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
[System.Xml.Serialization.XmlRootAttribute(Namespace="http://tempuri.org/amqp_definitions", IsNullable=false)]
public partial class definition {
    
    private doc[] docField;
    
    private string nameField;
    
    private string valueField;
    
    private string labelField;
    
    /// <remarks/>
    [System.Xml.Serialization.XmlElementAttribute("doc")]
    public doc[] doc {
        get {
            return this.docField;
        }
        set {
            this.docField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string name {
        get {
            return this.nameField;
        }
        set {
            this.nameField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string value {
        get {
            return this.valueField;
        }
        set {
            this.valueField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string label {
        get {
            return this.labelField;
        }
        set {
            this.labelField = value;
        }
    }
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Diagnostics.DebuggerStepThroughAttribute()]
[System.ComponentModel.DesignerCategoryAttribute("code")]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
[System.Xml.Serialization.XmlRootAttribute(Namespace="http://tempuri.org/amqp_definitions", IsNullable=false)]
public partial class type {
    
    private object[] itemsField;
    
    private string nameField;
    
    private typeClass classField;
    
    private string sourceField;
    
    private string providesField;
    
    private string labelField;
    
    /// <remarks/>
    [System.Xml.Serialization.XmlElementAttribute("choice", typeof(choice))]
    [System.Xml.Serialization.XmlElementAttribute("descriptor", typeof(descriptor))]
    [System.Xml.Serialization.XmlElementAttribute("doc", typeof(doc))]
    [System.Xml.Serialization.XmlElementAttribute("encoding", typeof(encoding))]
    [System.Xml.Serialization.XmlElementAttribute("field", typeof(field))]
    public object[] Items {
        get {
            return this.itemsField;
        }
        set {
            this.itemsField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string name {
        get {
            return this.nameField;
        }
        set {
            this.nameField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public typeClass @class {
        get {
            return this.classField;
        }
        set {
            this.classField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string source {
        get {
            return this.sourceField;
        }
        set {
            this.sourceField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string provides {
        get {
            return this.providesField;
        }
        set {
            this.providesField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string label {
        get {
            return this.labelField;
        }
        set {
            this.labelField = value;
        }
    }
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Diagnostics.DebuggerStepThroughAttribute()]
[System.ComponentModel.DesignerCategoryAttribute("code")]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
[System.Xml.Serialization.XmlRootAttribute(Namespace="http://tempuri.org/amqp_definitions", IsNullable=false)]
public partial class choice {
    
    private doc[] docField;
    
    private string nameField;
    
    private string valueField;
    
    /// <remarks/>
    [System.Xml.Serialization.XmlElementAttribute("doc")]
    public doc[] doc {
        get {
            return this.docField;
        }
        set {
            this.docField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string name {
        get {
            return this.nameField;
        }
        set {
            this.nameField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string value {
        get {
            return this.valueField;
        }
        set {
            this.valueField = value;
        }
    }
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Diagnostics.DebuggerStepThroughAttribute()]
[System.ComponentModel.DesignerCategoryAttribute("code")]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
[System.Xml.Serialization.XmlRootAttribute(Namespace="http://tempuri.org/amqp_definitions", IsNullable=false)]
public partial class descriptor {
    
    private doc[] docField;
    
    private string nameField;
    
    private string codeField;
    
    /// <remarks/>
    [System.Xml.Serialization.XmlElementAttribute("doc")]
    public doc[] doc {
        get {
            return this.docField;
        }
        set {
            this.docField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string name {
        get {
            return this.nameField;
        }
        set {
            this.nameField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string code {
        get {
            return this.codeField;
        }
        set {
            this.codeField = value;
        }
    }
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Diagnostics.DebuggerStepThroughAttribute()]
[System.ComponentModel.DesignerCategoryAttribute("code")]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
[System.Xml.Serialization.XmlRootAttribute(Namespace="http://tempuri.org/amqp_definitions", IsNullable=false)]
public partial class encoding {
    
    private doc[] docField;
    
    private string nameField;
    
    private string labelField;
    
    private string codeField;
    
    private encodingCategory categoryField;
    
    private string widthField;
    
    /// <remarks/>
    [System.Xml.Serialization.XmlElementAttribute("doc")]
    public doc[] doc {
        get {
            return this.docField;
        }
        set {
            this.docField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string name {
        get {
            return this.nameField;
        }
        set {
            this.nameField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string label {
        get {
            return this.labelField;
        }
        set {
            this.labelField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string code {
        get {
            return this.codeField;
        }
        set {
            this.codeField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public encodingCategory category {
        get {
            return this.categoryField;
        }
        set {
            this.categoryField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string width {
        get {
            return this.widthField;
        }
        set {
            this.widthField = value;
        }
    }
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
public enum encodingCategory {
    
    /// <remarks/>
    @fixed,
    
    /// <remarks/>
    variable,
    
    /// <remarks/>
    compound,
    
    /// <remarks/>
    array,
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Diagnostics.DebuggerStepThroughAttribute()]
[System.ComponentModel.DesignerCategoryAttribute("code")]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
[System.Xml.Serialization.XmlRootAttribute(Namespace="http://tempuri.org/amqp_definitions", IsNullable=false)]
public partial class field {
    
    private doc[] docField;
    
    private string nameField;
    
    private string typeField;
    
    private string requiresField;
    
    private string defaultField;
    
    private string labelField;
    
    private string mandatoryField;
    
    private string multipleField;
    
    /// <remarks/>
    [System.Xml.Serialization.XmlElementAttribute("doc")]
    public doc[] doc {
        get {
            return this.docField;
        }
        set {
            this.docField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string name {
        get {
            return this.nameField;
        }
        set {
            this.nameField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string type {
        get {
            return this.typeField;
        }
        set {
            this.typeField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string requires {
        get {
            return this.requiresField;
        }
        set {
            this.requiresField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string @default {
        get {
            return this.defaultField;
        }
        set {
            this.defaultField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string label {
        get {
            return this.labelField;
        }
        set {
            this.labelField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string mandatory {
        get {
            return this.mandatoryField;
        }
        set {
            this.mandatoryField = value;
        }
    }
    
    /// <remarks/>
    [System.Xml.Serialization.XmlAttributeAttribute()]
    public string multiple {
        get {
            return this.multipleField;
        }
        set {
            this.multipleField = value;
        }
    }
}

/// <remarks/>
[System.CodeDom.Compiler.GeneratedCodeAttribute("xsd", "4.0.30319.33440")]
[System.SerializableAttribute()]
[System.Xml.Serialization.XmlTypeAttribute(AnonymousType=true, Namespace="http://tempuri.org/amqp_definitions")]
public enum typeClass {
    
    /// <remarks/>
    primitive,
    
    /// <remarks/>
    composite,
    
    /// <remarks/>
    restricted,
    
    /// <remarks/>
    union,
}
