// Copyright (c) Microsoft. All rights reserved.

#define read_uint8_t real_read_uint8_t
#define read_uint16_t real_read_uint16_t
#define read_uint32_t real_read_uint32_t
#define read_uint64_t real_read_uint64_t

#define read_int8_t real_read_int8_t
#define read_int16_t real_read_int16_t
#define read_int32_t real_read_int32_t
#define read_int64_t real_read_int64_t

#define read_uuid_t real_read_uuid_t

#define write_uint8_t real_write_uint8_t
#define write_uint16_t real_write_uint16_t
#define write_uint32_t real_write_uint32_t
#define write_uint64_t real_write_uint64_t

#define write_int8_t real_write_int8_t
#define write_int16_t real_write_int16_t
#define write_int32_t real_write_int32_t
#define write_int64_t real_write_int64_t

#define write_uuid_t real_write_uuid_t
