// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

#ifndef GB_RAND_H
#define GB_RAND_H

#include "umock_c/umock_c_prod.h"

#ifdef __cplusplus
extern "C" {
#endif

MOCKABLE_FUNCTION(, int, gb_rand);

#ifdef __cplusplus
}
#endif

#endif /* GB_RAND_H */
