// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

#ifndef MU_DEFINE_ENUM_TEST_H
#define MU_DEFINE_ENUM_TEST_H

#include "azure_macro_utils/macro_utils.h"

int run_mu_define_enum_tests(void);

#endif // MU_DEFINE_ENUM_TEST_H
