// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

#ifndef UMOCKSTRING_H
#define UMOCKSTRING_H

#ifdef __cplusplus
extern "C" {
#endif

char* umockstring_clone(const char* source);

#ifdef __cplusplus
}
#endif

#endif /* UMOCKSTRING_H */
